==> python-language-24.2.0-sources.jar: com/oracle/graal/python/builtins/objects/exception/ChainExceptionsNode.java <==
/*
 * Copyright (c) 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.builtins.objects.exception;

import com.oracle.graal.python.builtins.objects.PNone;
import com.oracle.graal.python.runtime.exception.PException;
import com.oracle.truffle.api.dsl.Bind;
import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.GenerateInline;
import com.oracle.truffle.api.dsl.NeverDefault;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.api.profiles.InlinedLoopConditionProfile;

@GenerateInline(false) // Used in BCI
public abstract class ChainExceptionsNode extends Node {
    public abstract void execute(PException currentException, PException contextException);

    @Specialization
    public static void chainExceptions(PException currentException, PException contextException,
                    @Bind("this") Node inliningTarget,
                    @Cached ExceptionNodes.GetContextNode getContextNode,
                    @Cached ExceptionNodes.SetContextNode setContextNode,
                    @Cached InlinedLoopConditionProfile p1,
                    @Cached InlinedLoopConditionProfile p2) {
        Object current = currentException.getUnreifiedException();
        Object context = contextException.getUnreifiedException();
        if (current != context) {
            Object e = current;
            while (p1.profile(inliningTarget, e != PNone.NONE)) {
                Object eContext = getContextNode.execute(inliningTarget, e);
                if (eContext == context) {
                    // We have already chained this exception in an inner block, do nothing
                    return;
                }
                e = eContext;
            }
            e = context;
            while (p2.profile(inliningTarget, e != PNone.NONE)) {
                Object eContext = getContextNode.execute(inliningTarget, e);
                if (eContext == current) {
                    setContextNode.execute(inliningTarget, e, PNone.NONE);
                }
                e = eContext;
            }
            contextException.markEscaped();
            setContextNode.execute(inliningTarget, current, context);
        }
    }

    @NeverDefault
    public static ChainExceptionsNode create() {
        return ChainExceptionsNodeGen.create();
    }
}

==> python-language-25.0.2-sources.jar: com/oracle/graal/python/builtins/objects/exception/ChainExceptionsNode.java <==
/*
 * Copyright (c) 2023, 2025, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.builtins.objects.exception;

import com.oracle.graal.python.builtins.objects.PNone;
import com.oracle.graal.python.runtime.exception.PException;
import com.oracle.truffle.api.dsl.Bind;
import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.GenerateInline;
import com.oracle.truffle.api.dsl.NeverDefault;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.api.profiles.InlinedLoopConditionProfile;

@GenerateInline(false) // Used in BCI
public abstract class ChainExceptionsNode extends Node {
    public abstract void execute(PException currentException, PException contextException);

    @Specialization
    public static void chainExceptions(PException currentException, PException contextException,
                    @Bind Node inliningTarget,
                    @Cached ExceptionNodes.GetContextNode getContextNode,
                    @Cached ExceptionNodes.SetContextNode setContextNode,
                    @Cached InlinedLoopConditionProfile p1,
                    @Cached InlinedLoopConditionProfile p2) {
        Object current = currentException.getUnreifiedException();
        Object context = contextException.getUnreifiedException();
        if (current != context) {
            Object e = current;
            while (p1.profile(inliningTarget, e != PNone.NONE)) {
                Object eContext = getContextNode.execute(inliningTarget, e);
                if (eContext == context) {
                    // We have already chained this exception in an inner block, do nothing
                    return;
                }
                e = eContext;
            }
            e = context;
            while (p2.profile(inliningTarget, e != PNone.NONE)) {
                Object eContext = getContextNode.execute(inliningTarget, e);
                if (eContext == current) {
                    setContextNode.execute(inliningTarget, e, PNone.NONE);
                }
                e = eContext;
            }
            contextException.markEscaped();
            setContextNode.execute(inliningTarget, current, context);
        }
    }

    @NeverDefault
    public static ChainExceptionsNode create() {
        return ChainExceptionsNodeGen.create();
    }
}
